package com.yidian.shenghuoquan.border.ui

import android.os.Bundle
import androidx.activity.viewModels
import androidx.recyclerview.widget.LinearLayoutManager
import com.yidian.bcommon.constant.IntentConstants
import com.yidian.bcommon.constant.XEventConfig
import com.yidian.bcommon.constant.XRouterPathConstants
import com.yidian.bcommon.mvvm.BaseMvvmActivity
import com.yidian.shenghuoquan.border.R
import com.yidian.shenghuoquan.border.adapter.PickupPointListAdapter
import com.yidian.shenghuoquan.border.databinding.BorderActivitySelectPickupPointBinding
import com.yidian.shenghuoquan.border.dto.PickupPointListDto
import com.yidian.shenghuoquan.border.viewmodel.SelectPickupPointViewModel
import com.yidian.xarc.xevent.XBaseEvent
import com.yidian.xarc.xevent.XEventManager
import com.yidian.xpage.XPageViewProtocol
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

/**
 * author: yinjiacheng
 * date: 2021/8/24 14:02
 * description: 选择自提点页
 */
class SelectPickupPointActivity : BaseMvvmActivity<BorderActivitySelectPickupPointBinding, SelectPickupPointViewModel>(),
  XPageViewProtocol {

  override val layoutId = R.layout.border_activity_select_pickup_point
  override val vm: SelectPickupPointViewModel by viewModels()
  private val adapter by lazy { PickupPointListAdapter(null) }
  private val params by lazy {
    intent.getSerializableExtra(XRouterPathConstants.ParamsKey)?.let {
      it as HashMap<*, *>
    }
  }

  /**
   * 搜索关键字
   */
  private var keyWords: String? = null

  override fun getXPageName(): String {
    return XRouterPathConstants.PICKUP_POINT_ACTIVITY
  }

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    XEventManager.register(this)
    // 标记是否有当前位置
    binding.location = params?.get(IntentConstants.KEY_EXTRA_LONGITUDE) != null && params?.get(IntentConstants.KEY_EXTRA_LATITUDE) != null
  }

  override fun onDestroy() {
    super.onDestroy()
    XEventManager.unRegister(this)
  }

  override fun initView() {
    initRecyclerView()
    initSmartRefresh()
    registerUIEvent()
    requestPickupPoint(SelectPickupPointViewModel.LoadType.SEARCH)
  }

  private fun initRecyclerView() {
    binding.rvPickupPointList.adapter = adapter
    binding.rvPickupPointList.layoutManager = LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
    binding.rvPickupPointList.addItemDecoration(PickupPointListAdapter.PickupPointListItemDecoration())
  }

  private fun initSmartRefresh() {
    binding.rlPickupPointList.setOnRefreshListener {
      requestPickupPoint(SelectPickupPointViewModel.LoadType.REFRESH)
    }
    binding.rlPickupPointList.setOnLoadMoreListener {
      requestPickupPoint(SelectPickupPointViewModel.LoadType.LOAD_MORE)
    }
  }

  /**
   * 注册UI事件监听
   */
  private fun registerUIEvent() {
    vm.pickupPointListSearch.observe(this, {
      updateData(it)
    })
    vm.pickupPointListRefresh.observe(this, {
      binding.rlPickupPointList.finishRefresh(it != null)
      updateData(it)
    })
    vm.pickupPointListLoadMore.observe(this, {
      binding.rlPickupPointList.finishLoadMore(it != null)
      updateData(it)
    })
  }

  /**
   * 更新列表数据
   */
  private fun updateData(data: PickupPointListDto?) {
    if (data == null) return
    // 标记当前选中的自提点
    adapter.setSelectedId(params?.get(IntentConstants.KEY_EXTRA_PICKUP_POINT_ID)?.let { id -> id as Long })
    adapter.updateData(data.list)
  }

  /**
   * 请求自提点列表
   */
  private fun requestPickupPoint(type: SelectPickupPointViewModel.LoadType) {
    vm.requestPickupPointList(
      params?.get(IntentConstants.KEY_EXTRA_MARKETING_ID) as String,
      params?.get(IntentConstants.KEY_EXTRA_LONGITUDE)?.let { it as Double },
      params?.get(IntentConstants.KEY_EXTRA_LATITUDE)?.let { it as Double },
      keyWords, type
    )
  }

  @Subscribe(sticky = false, threadMode = ThreadMode.MAIN)
  fun onEvent(event: XBaseEvent?) {
    when (event?.name) {
      XEventConfig.SEARCH_PICKUP_POINT_CANCEL -> {
        keyWords = null
        // 取消搜索自提点 请求附近自提点
        requestPickupPoint(SelectPickupPointViewModel.LoadType.SEARCH)
      }
      XEventConfig.SEARCH_PICKUP_POINT_START -> {
        keyWords = event.body as String
        // 开始搜索自提点
        requestPickupPoint(SelectPickupPointViewModel.LoadType.SEARCH)
      }
    }
  }
}
